# -*- coding: utf-8 -*-
"""
Created on Sat May 23 07:40:01 2020

@author: Bayron
"""

import seaborn as sns
import pandas as pd
import matplotlib.pyplot as plt
import seawolf as sw
import time

url = "https://raw.github.com/mattdelhey/kaggle-titanic/master/Data/train.csv"
titanic = pd.read_csv(url)
# print(titanic.columns)

sw.style.set(figsize=(8, 6))

ax = sns.countplot(data=titanic, y="pclass", hue='sex')
sw.show_values( minvalue=100, maxvalue=200,
    ax=ax, dec=0, color="black", fontweight="bold", loc="top"
)
# sw.set_values(dec=1, values=[1,2,3,4,5,6], color='red')
sw.set_legend(
    title="Sexo",
    ncols=1,
    labels=["Hombre", "Mujer"],
    label_fontsize=8,
    title_fontsize=10,
    title_loc="center",
    borderpad=1.22,
)
sw.set_title(title="Tabla 3", fontweight="bold")
sw.set_subtitle(title="Conteo de sobrevivientes")
sw.set_tickslabel(
    axis="y",
    labelrotation=0,
    loc='in',
    xpad=0,
    ypad=0,
    color="blue",
    # colors = ['white', 'red', 'white'],
    shadow=0.5,
    shadowcolor="black",
    labels=["Baja", "Media", "Alta"],
)
sw.theme(op='spine', left=True, bottom=True, spine_butt='left')
ax.xaxis.grid(True)
ax.set_axisbelow(True)
plt.show()


# -*- coding: utf-8 -*-
"""
Created on Sun May 24 05:58:23 2020

@author: Bayron Torres
"""
import matplotlib as _mpl
import matplotlib.patheffects as path_effects
import numpy as _np
import seaborn as _sns
from matplotlib import pyplot as _plt
from matplotlib.axis import Tick

from ._axesTools import _axTools
from ._style import StyleSW as style

# =============================================================================
# Principal Axis class (implements new methods here)
# =============================================================================


def set_axislabel(
    ax=None, xlabel: str = str(), ylabel: str = str(), **kwargs
) -> _plt.Axes:
    ax = _axTools.gca(ax)
    # Getting initial vars
    d, kwargs = _axTools.get_init_kwargs(kwargs, defaults={"fontweight": "bold"})

    # Setting font style
    font = {"color": d["color"], "fontweight": d["fontweight"], "size": d["fontsize"]}

    if xlabel is not None:
        xlabel = xlabel.strip()
    if ylabel is not None:
        ylabel = ylabel.strip()
    # If not FaceGrid plot
    if isinstance(ax, _sns.axisgrid.FacetGrid):
        # If is FaceGrid plot
        if isinstance(ax.axes[0], _mpl.axes.Axes):
            ncols = ax.axes[0].get_subplotspec().get_gridspec().get_geometry()[1]
            tam_grp = len(ax.axes)
            is_axes = True
            iterate_group = ax.axes
        else:
            is_axes = False
            iterate_group = ax.axes[0]
        xlabel_ = iterate_group[0].get_xlabel()
        ylabel_ = iterate_group[0].get_ylabel()
        # Iter for each plot
        for i, a in enumerate(iterate_group):
            if xlabel is None:
                xlabel_ = ""
            elif len(xlabel) > 0:
                xlabel_ = xlabel
            if ylabel is None:
                ylabel_ = ""
            elif len(ylabel) > 0:
                ylabel_ = ylabel
            if is_axes:
                if (i + 1 + ncols) > tam_grp:
                    xlabel_ = xlabel
                else:
                    xlabel_ = ""
                if i % ncols != 0:
                    ylabel_ = ""
            else:
                if i > 0:
                    ylabel_ = ""
            a.set_xlabel(xlabel_, fontdict=font, labelpad=d["xpad"])
            a.set_ylabel(ylabel_, fontdict=font, labelpad=d["ypad"])
    else:
        if xlabel is None:
            xlabel = ""
        elif len(xlabel) == 0:
            xlabel = ax.get_xlabel()
        if ylabel is None:
            ylabel = ""
        elif len(ylabel) == 0:
            ylabel = ax.get_ylabel()
        ax.set_xlabel(xlabel, fontdict=font, labelpad=d["xpad"])
        ax.set_ylabel(ylabel, fontdict=font, labelpad=d["ypad"])
    return ax


def get_axislabel(ax=None) -> tuple:
    ax = _axTools.gca(ax)
    x = ax.get_xlabel()
    y = ax.get_ylabel()
    return x, y


def get_tickslabel(ax=None) -> tuple:
    ax = _axTools.gca(ax)
    y = [y.get_text() for y in ax.get_yticklabels()]
    x = [x.get_text() for x in ax.get_xticklabels()]
    return x, y


def set_tickslabel(
    ax=None,
    axis: str = "x",
    visible: bool = True,
    labels: list = (),
    labelrotation: int = 0,
    loc=None,
    limit=_np.Infinity,
    bgcolors=list(),
    **kwargs,
) -> _plt.Axes:
    ax = _axTools.gca(ax)

    if labels is None:
        labels = list()

    if loc == "None":
        if axis == "x":
            def_dic = {"va": "top", "ha": "center"}
        elif axis == "y":
            def_dic = {"va": "center", "ha": "right"}
    else:
        def_dic = {"va": "center_baseline", "ha": "center"}

    d, kwargs = _axTools.get_init_kwargs(kwargs, defaults=def_dic)
    print(kwargs)

    locs = ["in", "on", "bottom", "top", "left", "right", None]
    if not loc in locs:
        raise ValueError("The value in LOC is not recognized: {0}".format(locs))

    orient, _ = _axTools.orient(ax)

    fig = ax.get_figure()
    fig.canvas.draw()

    ax.tick_params(
        grid_color=d["grid_color"],
        color=d["color"],
        grid_alpha=d["grid_alpha"],
        labelsize=d["fontsize"],
        axis=axis,
        labelrotation=labelrotation,
        **kwargs,
    )

    if axis == "x":
        labels_ax = ax.get_xticklabels()
    elif axis == "y":
        labels_ax = ax.get_yticklabels()
    else:
        raise ValueError("axis value {0} is not available".format(axis))

    if len(labels) == 0:
        labels = [l.get_text() for l in labels_ax]
    elif len(labels) < len(labels_ax):
        for i in range(0, len(labels_ax) - len(labels)):
            labels.append(None)

    if len(bgcolors) > 0:
        bbox = dict(boxstyle="square", alpha=0.3)
        for i, c in enumerate(bgcolors):
            labels_ax[i].set_bbox(bbox)
            labels_ax[i].set_backgroundcolor(c)

    if isinstance(d["colors"], list):
        colors = d["colors"]
    else:
        colors = _axTools.getColorList(len(labels), d["colors"])

    va_text = d["va"]
    ha_text = d["ha"]

    if axis == "both":
        if visible == False:
            ax.axes.xaxis.set_visible(False)
            ax.axes.yaxis.set_visible(False)
            return
    else:
        if axis == "x":
            if visible == False:
                ax.axes.xaxis.set_visible(False)
                return
            else:
                ax.axes.xaxis.set_visible(True)
                ax.set_xticklabels(
                    labels,
                    rotation=labelrotation,
                    ha=ha_text,
                    va=va_text,
                    weight=d["fontweight"],
                )

        if axis == "y":
            if visible == False:
                ax.axes.yaxis.set_visible(False)
                return
            else:
                ax.axes.yaxis.set_visible(True)
                ax.set_yticklabels(
                    labels,
                    rotation=labelrotation,
                    ha=ha_text,
                    va=va_text,
                    weight=d["fontweight"],
                )
    # NO DIRECTION
    if loc is None:
        if axis == "x":
            ax.set_xticks(ax.get_xticks())
            for color, tick in zip(colors, ax.xaxis.get_major_ticks()):
                tick.label1.set_color(color)

        if axis == "y":
            ax.set_yticks(ax.get_yticks())
            for color, tick in zip(colors, ax.yaxis.get_major_ticks()):
                tick.label1.set_color(color)

    else:
        if len(colors) < len(labels):
            _aux_ = _axTools.getColorList((len(labels) - len(colors)), d["color"])
            colors.extend(_aux_)

        def get_data():
            data = _axTools.values_bars(ax)
            mask = data["value"] >= limit
            data.loc[mask, "value"] = 0
            return data

        values_iterables = None

        if loc in ["bottom", "in", "on"]:
            data = get_data()
            posY = data["value"]

            if axis == "x":
                posX = [x.get_position()[0] for x in labels_ax]
            else:
                posX = [x.get_position()[1] for x in labels_ax]

            if loc == "bottom":
                if len(labels) != len(posY):
                    posY = [0] * data.shape[0]

            else:
                if loc in ["in", "on"]:
                    if axis == "x":
                        posX = data["x"]
                    else:
                        posX = data["y"]

                    if loc == "in":
                        posY = [x / 2 for x in posY]

                    if len(labels) != len(posY):
                        factor = int(len(posY) / len(labels))
                        labels = labels * factor
                        colors = colors * factor

            if axis == "x":
                ax.set_xticklabels([])
                values_iterables = zip(labels, posX, posY, colors)
            elif axis == "y":
                ax.set_yticklabels([])
                values_iterables = zip(labels, posY, posX, colors)

        else:
            if orient == "v":
                if axis == "y":
                    if loc == "right":
                        ax.yaxis.set_ticks_position("right")
                        ax.yaxis.set_label_position("right")
                    elif loc == "left":
                        ax.yaxis.set_ticks_position("left")
                        ax.yaxis.set_label_position("left")
                    elif loc == "top":
                        print(
                            "Loc '{0}' is not available in axis {1}".format(loc, axis)
                        )

                elif axis == "x":
                    if loc in ["left", "right"]:
                        print(
                            "Loc '{0}' is not available in axis {1}".format(loc, axis)
                        )
                    elif loc == "top":
                        ax.xaxis.set_ticks_position("top")
                        ax.xaxis.set_label_position("top")

            elif orient == "h":
                if axis == "x":
                    if loc == "top":
                        ax.xaxis.set_ticks_position("top")
                        ax.xaxis.set_label_position("top")
                    else:
                        print(
                            "Loc '{0}' is not available in axis {1}".format(loc, axis)
                        )
                elif axis == "y":
                    if loc in ["right", "top"]:
                        ax.yaxis.set_ticks_position("right")
                        ax.yaxis.set_label_position("right")
                    elif loc == "left":
                        ax.yaxis.set_ticks_position("left")
                        ax.yaxis.set_label_position("left")

        if not values_iterables is None:
            effects = [
                path_effects.Stroke(
                    linewidth=d["shadow"],
                    foreground=d["shadowcolor"],
                    alpha=0.8,
                ),
                path_effects.Normal(),
            ]

            for name, x_pos, y_pos, c in values_iterables:
                if orient == "h":
                    PADT = -1 * d["xpad"] if x_pos < 0 else d["xpad"]
                    if x_pos >= 0:
                        ha_text = "left"
                    elif x_pos < 0:
                        ha_text = "right"
                    x_pos = x_pos + PADT
                    y_pos = y_pos + d["ypad"]
                else:
                    PADT = -1 * d["ypad"] if y_pos < 0 else d["ypad"]
                    if x_pos >= 0:
                        va_text = "bottom"
                    elif x_pos < 0:
                        va_text = "top"
                    y_pos = y_pos + PADT
                    x_pos = x_pos + d["xpad"]

                t = ax.text(
                    x_pos,
                    y_pos,
                    name,
                    va=va_text,
                    rotation=labelrotation,
                    color=c,
                    fontsize=d["fontsize"],
                    ha=ha_text,
                    weight=d["fontweight"],
                    path_effects=effects,
                )

                if d["shadow"] > 0:
                    t.set_path_effects(effects)
    return ax


def theme(
    op="spine",
    top: bool = False,
    right: bool = False,
    left: bool = False,
    bottom: bool = False,
    despine_trim: bool = False,
    despine_offset: int = 0,
    spine_butt="left",
    ax=None,
):
    ax = _axTools.gca(ax)
    options = ["despine", "spine", "clear"]
    if not op in options:
        raise ValueError("Value is not in '{0}'".format(options))

    if op == options[0]:
        _sns.despine(
            top=top,
            right=right,
            left=left,
            bottom=bottom,
            trim=despine_trim,
            offset=despine_offset,
        )

    elif op == options[1]:
        ax.spines["left"].set_visible(left)
        ax.spines["top"].set_visible(top)
        ax.spines["bottom"].set_visible(bottom)
        ax.spines["right"].set_visible(right)
        ax.spines[spine_butt].set_lw(1.5)
        ax.spines[spine_butt].set_capstyle("butt")

    elif op == options[2]:
        orient = _axTools.orient(ax)
        _sns.despine(top=True, bottom=True, right=True, left=True, trim=True, offset=3)
        if orient == "v":
            ax.get_yaxis().set_visible(False)
        elif orient == "h":
            ax.get_xaxis().set_visible(False)
        set_axislabel(xlabel=None, ylabel=None)

    else:
        _sns.despine(top=True, bottom=False, right=False, left=False, trim=False)

    return ax


def set_title(ax=None, title: str = str(), loc: str = "left", **kwargs) -> None:
    d, kwargs = _axTools.get_init_kwargs(
        kwargs,
        defaults={
            "fontweight": "bold",
            "color": "black",
            "fontsize": _mpl.rcParams["figure.titlesize"],
        },
    )

    if loc == "center":
        pos = 0.5
    elif loc == "left":
        pos = 0.0
    elif loc == "right":
        pos = 0.96
    else:
        pos = 0.0
        loc = "left"

    if ax is None:
        ax = _axTools.gca(ax)

    # Check if set figure or Axesplot
    if isinstance(ax, _mpl.axes._subplots.SubplotBase):
        ax = _axTools.gca(ax)
        ax.set_title(
            label=title,
            x=pos + d["xpad"],
            y=1.05 + d["ypad"],
            verticalalignment="bottom",
            horizontalalignment=loc,
            color=d["color"],
            fontweight=d["fontweight"],
            fontsize=d["fontsize"],
        )

    elif isinstance(ax, _mpl.figure.Figure):
        _plt.suptitle(
            title,
            y=1 + d["ypad"],
            x=pos + d["xpad"],
            verticalalignment="bottom",
            horizontalalignment=loc,
            fontsize=d["fontsize"],
            fontweight=d["fontweight"],
            color=d["color"],
        )


def set_subtitle(ax=None, title: str = str(), loc: str = "left", **kwargs) -> None:
    ax = _axTools.gca(ax)
    d, kwargs = _axTools.get_init_kwargs(kwargs)

    ad = 0.04 if (isinstance(ax, _mpl.figure.Figure)) else 0
    if loc == "center":
        pos = 0.50
    elif loc == "left":
        pos = 0 + ad
    elif loc == "right":
        pos = 1 - ad
    else:
        pos = 0.048
        loc = "left"

    if isinstance(ax, _mpl.axes._subplots.SubplotBase):
        txt = ax.text(
            x=pos + d["xpad"],
            y=1.02 + d["ypad"],
            s=title,
            transform=ax.transAxes,
            fontsize=d["fontsize"],
            fontweight=d["fontweight"],
            horizontalalignment=loc,
            verticalalignment="bottom",
        )
        _plt.setp(txt, color=d["color"])
    elif isinstance(ax, _mpl.figure.Figure):
        ax.text(
            s=title,
            x=pos + d["xpad"],
            y=1 + d["ypad"],
            fontsize=d["fontsize"],
            fontweight=d["fontweight"],
            color=d["color"],
            horizontalalignment=loc,
            verticalalignment="bottom",
        )


def set_legend(
    ax=None,
    show: bool = True,
    title: str = None,
    ncols: int = 1,
    loc=0,
    title_fontsize=None,
    title_loc: str = "left",
    label_fontsize=None,
    labels: list() = None,
    handles: list() = None,
    borderpad=0.82,
    **kwargs,
) -> _plt.Axes.legend:
    legend = None

    if type(loc) is tuple:
        bbox_to_anchor = loc
        loc = 0
    else:
        bbox_to_anchor = None

    if label_fontsize is None:
        label_fontsize = _mpl.rcParams["legend.fontsize"]
    if title_fontsize is None:
        title_fontsize = _mpl.rcParams["legend.fontsize"]

    propF = {"weight": "normal", "size": label_fontsize}

    if not isinstance(ax, _mpl.figure.Figure):
        ax = _axTools.gca(ax)

        if show:
            leg = ax.legend()
            leg = ax.legend(["Graph 1"]) if leg is not None else leg

            if title == "" and ax.get_legend() != None:
                title = ax.get_legend().get_title().get_text()
                title = title.capitalize()
            elif title is None:
                ax.legend_.set_title(None)

            if handles is None or labels is None:
                handles1, legs1 = ax.get_legend_handles_labels()
                handles = handles1 if handles is None else handles
                labels = legs1 if labels is None else labels
                for h, l in zip(handles, labels):
                    h.set_label(l)

            if not handles:
                print("No legend to show")
        else:
            # Remove legend
            l = ax.get_legend()
            if l is not None:
                l.remove()
    else:
        axs = list()
        loc = 8 if loc == 0 else loc

        if ax is None:
            axs = ax.axes
        else:
            if isinstance(ax, list):
                axs.extend(ax)
            else:
                axs.append(ax)
            for a in ax.axes:
                leg = a.legend(["Graph 1"])
                leg.remove()

        handles, legs = list(), list()
        ax.legends = []
        for a in axs:
            h, l = a.get_legend_handles_labels()
            handles.extend(h)
            legs.extend(l)
        legs = labels if labels is not None else legs

    legend = ax.legend(
        title=title,
        handles=handles,
        prop=propF,
        labels=labels,
        loc=loc,
        title_fontsize=title_fontsize,
        borderpad=borderpad,
        ncol=ncols,
        bbox_to_anchor=bbox_to_anchor,
        **kwargs,
    )

    if not title_loc in ["left", "right", "center"]:
        title_loc = "left"
    legend._legend_box.align = title_loc
    legend.get_title().set_position((0, 5))

    return legend


def get_values(
    ax=None, kind: str = "bar", normalized: bool = False, normBy="c", stack=False
):
    ax = _axTools.gca(ax)
    data = None
    if kind == "bar":
        data = _axTools.values_bars(ax=ax, normBy=normBy, stack=stack)
    elif kind == "line":
        data = _axTools.values_line(ax=ax, orient="v")
    elif kind == "point":
        data = _axTools.values_points(ax=ax)
    if normalized and data is not None:
        return _axTools.normalize_data(data)
    else:
        return data


def show_values(
    ax=None,
    dec: int = 1,
    minvalue: int = -_np.Infinity,
    maxvalue: int = _np.Infinity,
    loc: str = "top",
    kind: str = "bar",
    normBy: str = "c",
    display: str = "v",
    prefix: str = "",
    postfix: str = "",
    stack=False,
    **kwargs,
) -> _plt.Axes:
    ax = _axTools.gca(ax)
    orient = "v"

    d, kwargs = _axTools.get_init_kwargs(
        kwargs,
        remove_kwargs=False,
        inplace_kwargs=True,
        defaults={"va": "auto", "ha": "auto"},
    )

    if dec == None:
        dec = 2 if display != "v" else 1

    if kind not in ["bar", "line", "point"]:
        raise ValueError("kind is not in bar, line or point")

    # Bars
    if kind == "bar":
        orient, _ = _axTools.orient(ax)
        data = _axTools.values_bars(ax=ax, normBy=normBy, loc=loc, stack=stack)
        if display != "v":
            data = _axTools.normalize_data(data)

    # Lines shape
    elif kind == "line":
        data = _axTools.values_line(ax, "v")

        # For y values
        for u in data["index_color"].unique():
            df = data.loc[data["index_color"] == u]
            dfy = df["y"].to_numpy()
            dfy_len = len(dfy)
            new_dat = [0] * dfy_len
            for i in range(0, dfy_len):
                j = i + 1
                if i == dfy_len - 1:
                    j = i - 1
                if dfy[i] < dfy[j]:
                    new_dat[i] = dfy[i] - d["ypad"]
                else:
                    new_dat[i] = dfy[i] + d["ypad"]
            data.loc[data["index_color"] == u, "y"] = new_dat

        # Normalizing data
        if display != "v":
            data = _axTools.normalize_data(data)
    # Points shape
    elif kind == "point":
        data = _axTools.values_points(ax)
        # Normalizing data
        if display != "v":
            data = _axTools.normalize_data(data)

    # Add spacing for the data
    if kind != "line":
        if d["xpad"] != 0:
            data["x"] = [x + d["xpad"] if x >= 0 else x - d["xpad"] for x in data["x"]]
        if d["ypad"] != 0:
            data["y"] = [y + d["ypad"] if y >= 0 else y - d["ypad"] for y in data["y"]]

    # Print values on plot
    _axTools.plot_values(
        ax=ax,
        data=data,
        minvalue=minvalue,
        maxvalue=maxvalue,
        dec=dec,
        display=display,
        prefix=prefix,
        postfix=postfix,
        orient=orient,
        **kwargs,
    )


def set_values(
    ax=None,
    values=list(),
    dec: int = 1,
    minvalue: int = -_np.Infinity,
    maxvalue: int = _np.Infinity,
    loc: str = "top",
    kind: str = "bar",
    display: str = "v",
    stack=False,
    **kwargs,
):
    orient = "v"

    ax = _axTools.gca(ax)
    d, _ = _axTools.get_init_kwargs(
        kwargs=kwargs, remove_kwargs=False, inplace_kwargs=True
    )

    if values is not None or not isinstance(values, (list)):
        if kind == "bar":
            data = _axTools.values_bars(ax=ax, normBy="c", loc=loc, stack=stack)
            orient, _ = _axTools.orient(ax)
        elif kind == "line":
            data = _axTools.values_line(ax=ax, orient=orient)
        elif kind == "point":
            data = _axTools.values_points(ax=ax)

        # Add spacing for the data
        xpad = d["xpad"]
        ypad = d["ypad"]
        if kind != "line":
            if xpad != 0:
                data["x"] = [x + xpad if x >= 0 else x - xpad for x in data["x"]]
            if ypad != 0:
                data["y"] = [y + ypad if y >= 0 else y - ypad for y in data["y"]]

        if data.shape[0] >= len(values):
            n_items = len(values)
        else:
            n_items = data.shape[0]
        data["value"] = values[:n_items]

        _axTools.plot_values(
            ax=ax,
            data=data,
            minvalue=minvalue,
            maxvalue=maxvalue,
            dec=dec,
            display=display,
            orient=orient,
            **kwargs,
        )

    else:
        raise ValueError("`Values` data type is invalid")


def set_figsize(figsize):
    try:
        _plt.figure(figsize=figsize, dpi=80)
    except:
        raise ValueError("Figsize value has incorrect")


def get_figsize():
    return _mpl.rcParams["figure.figsize"]


def set_alpha(ax=None, alpha: float = 1.0):
    if ax is not None:
        if isinstance(ax, _mpl.axes.Axes):
            ch = ax.get_children()
            for c in ch:
                if isinstance(c, _axTools.artistList):
                    c.set_alpha(alpha)
                elif isinstance(c, _mpl.patches.Rectangle):
                    """All rectangles and not background rectangle"""
                    if (
                        not float(c.get_width()) == 1.0
                        and not float(c.get_height()) == 1.0
                    ):
                        c.set_alpha(alpha)



# -*- coding: utf-8 -*-
"""
Created on Wed Apr  1 06:08:47 2020

@author: Bayron
"""

import matplotlib as mpl
from os import listdir
from os.path import isfile, join
from matplotlib import font_manager
import os

# =============================================================================
# Set global custom style plot
# =============================================================================


class _styleBase(object):
    # Variables
    __axes_color__ = "#0B0201"
    __ticks_color__ = '#0B0201'
    __font_scale__ = 1.1
    __rc_plot__ = {'grid.linestyle': 'dashed',
                   'lines.linewidth': 2.8,
                   #    'axes.labelcolor': __axes_color__,
                   #    'axes.edgecolor': __ticks_color__,
                   #    'ytick.color': __ticks_color__,
                   #    'xtick.color': __ticks_color__,
                   'font.weight': 'normal'}
    __default_font__ = 'Cambria'


# =============================================================================
# Seawolf Style class
# =============================================================================


class StyleSW(_styleBase):

    def set(figsize=None, font_weight='normal', font_family=None, fontsize=None,
            rc: dict = None):

        if rc is None:
            rc = {}

        if figsize is None:
            mpl.rcParams['figure.figsize'] = (12, 10)
        elif type(figsize) is tuple:
            mpl.rcParams['figure.figsize'] = figsize

        if font_family is not None:
            rc['font.family'] = font_family

        for i, k in rc.items():
            _styleBase.__rc_plot__[i] = k

        if fontsize is None:
            fontsize = mpl.rcParams['figure.figsize'][0]
        elif fontsize > 10:
            _styleBase.__rc_plot__['font.size'] = fontsize * 0.85
            _styleBase.__rc_plot__['figure.titlesize'] = fontsize * 1.8
            _styleBase.__rc_plot__['ytick.labelsize'] = fontsize * 0.7
            _styleBase.__rc_plot__['xtick.labelsize'] = fontsize * 0.7
            _styleBase.__rc_plot__['axes.labelsize'] = fontsize * 0.7
            _styleBase.__rc_plot__['legend.fontsize'] = fontsize * 0.5
            _styleBase.__rc_plot__['legend.title_fontsize'] = fontsize * 0.65
        else:
            _styleBase.__rc_plot__['font.size'] = 10
            _styleBase.__rc_plot__['figure.titlesize'] = 'x-large'
            _styleBase.__rc_plot__['ytick.labelsize'] = 'medium'
            _styleBase.__rc_plot__['xtick.labelsize'] = 'medium'
            _styleBase.__rc_plot__['axes.labelsize'] = 'medium'
            _styleBase.__rc_plot__['legend.fontsize'] = 'small'

        _styleBase.__rc_plot__['font.weight'] = font_weight
        mpl.rcParams.update(_styleBase.__rc_plot__)

    def addPathFonts(font_path=''):
        onlyfiles = [f for f in listdir(
            font_path) if isfile(join(font_path, f))]
        for of in onlyfiles:
            font_manager.fontManager.addfont(font_path + of)
            prop = font_manager.FontProperties(fname=font_path + of)
            print(prop.get_name())

    def setFont(font_name: str = 'Cambria'):
        if font_name == _styleBase.__default_font__:
            StyleSW.addPathFonts(os.path.dirname(
                os.path.abspath(__file__)) + '\\fonts\\')
        mpl.rcParams['font.family'] = 'sans-serif'
        mpl.rcParams['font.sans-serif'] = font_name

    def reset_default():
        _styleBase.__rc_plot__
        mpl.rcdefaults()
        mpl.rcParams['figure.figsize'] = (12, 10)

    def get_axesColor():
        return _styleBase.__axes_color__

    def get_ticksColor():
        return _styleBase.__ticks_color__

    def get_figSize():
        return mpl.rcParams['figure.figsize']

    def set_style(name: str = 'seaborn'):
        lst_style = mpl.style.available
        if name is not None:
            if name in lst_style:
                a = mpl.style.context(name)
                return a
            else:
                raise NameError('Style names valid are: {0}'.format(lst_style))
        else:
            raise NameError('None type is not a valid style')

    def get_figsize():
        return mpl.rcParams['figure.figsize']



# -*- coding: utf-8 -*-
"""
Created on Mon Mar 25 09:57:59 2019

@author: bayron.torres
"""
import matplotlib as mpl

Category45 = ['#1B7256', '#379683', '#5CDB95', '#8EE4AF', '#8EE3D5',
              '#98D3E1', '#97CAEF', '#97BCEF', '#c293ed', '#E7717D',
              '#FC4445', '#C3073F', '#950740', '#6F2232', '#5D001E',
              '#b3476a', '#c9718e', '#edbbcc', '#C1C8E4', '#84CEEB',
              '#5AB9EA', '#5680E9', '#8860D0', '#A13E97', '#bd3ab1',
              '#c973bf', '#f075b7', '#FE929F', '#ffb3bc', '#fcdce0',
              '#FFD954', '#FEC804', '#F2AB39', '#CF9A41', '#CD7000',
              '#9B4F0F', '#CD5100', '#FF6602', '#F2A122', '#F2C022',
              '#F2DF22', '#E5F222', '#D8F573', '#B1F573', '#bdd6a7']

Category20 = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
              '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf',
              '#aec7e8', '#ffbb78', '#98df8a', '#ff9896', '#c5b0d5',
              '#c49c94', '#f7b6d2', '#c7c7c7', '#dbdb8d', '#9edae5']

Category10 = ['#01BEFE', '#F12000', '#1BCF28', '#FFDD00', '#8F00FF',
              '#F51899', '#FF7F0E', '#ADFF02', '#4081ED', '#FEA900']

Category6 = ['#01BEFE', '#FFDD00', '#FF8109', '#FF006D', '#ADFF02',
             '#8F00FF']

Colorblind = ['#0072b2', '#e69f00', '#f0e442', '#009e73', '#56b4e9', '#d55e00',
              '#cc79a7', 'black']

Dark = ['#1b9e77', '#d95f02', '#7570b3', '#e7298a', '#66a61e', '#e6ab02',
        '#a6761d', '#666666']

Pastel = ['#fbb4ae', '#b3cde3', '#ccebc5', '#decbe4', '#fed9a6', '#ffffcc',
          '#e5d8bd', '#fddaec', '#f2f2f2']

__all_list__ = {"cat45": Category45,
                "cat20": Category20,
                "cat10": Category10,
                "cat6": Category6,
                "colorblind": Colorblind,
                "dark": Dark,
                "pastel": Pastel}

# =============================================================================
# Set global custom style plot
# =============================================================================


class ColorsSW(object):

    def color_palette(palette=None, ini=0, n_colors=None):
        colors = list()
        rev = False

        if palette == None:
            palette = "cat10"
        elif palette.endswith("_r"):
            palette = palette[:-2]
            rev = True
        try:
            if palette in __all_list__:
                colors = __all_list__.get(palette)

            else:
                cmap = mpl.cm.get_cmap(palette)
                for i in range(cmap.N):
                    rgb = cmap(i)[:3]
                    colors.append(rgb)
        except ValueError as ex:
            print(str(ex) + " " +
                  "".join([", '" + x + "'" for x in __all_list__.keys()]))

        n = len(colors)
        if n_colors is None:
            n_colors = n
        if ini > n:
            raise ValueError('´´ini´´ value is not valid')

        colors = colors[ini: (ini+n_colors)]
        if rev:
            colors.reverse()
        return colors

    def hex_to_rgb(value):
        value = value.lstrip('#')
        lv = len(value)
        return tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))

    def rgb_to_hex(rgb):
        return '#%02x%02x%02x' % rgb

    def contrastColor(color):
        varHex = False
        if type(color) == str:
            varHex = True
            if color.find('#') == False:
                color = '#' + color
            color = ColorsSW.hex_to_rgb(color)

        if (type(color) == tuple):
            if len(color) == 3:
                contrast = (255-color[0], 255-color[1], 255-color[2])
                if (varHex):
                    contrast = ColorsSW.rgb_to_hex(contrast)
                return contrast
            else:
                raise ValueError('RGB color value is not valid')
        else:
            raise ValueError('COLOR value is not valid')



# =============================================================================
# Tools for Axes manage
# =============================================================================
import math
import numpy as np
from operator import contains
from string import Template

import matplotlib as _mpl
import matplotlib.patheffects as path_effects
import matplotlib.pyplot as _plt
import numpy as _np
import pandas as _pd
import seaborn as _sns

from ._style import StyleSW as style


class _axTools(object):
    artistList = (
        _mpl.patches.Wedge,
        _mpl.patches.Ellipse,
        _mpl.patches.Circle,
        _mpl.patches.Shadow,
        _mpl.lines.Line2D,
        _mpl.collections.PathCollection,
        _mpl.collections.PolyCollection,
    )

    def gca(ax):
        """
         Set the current axes. This is a convenience function to make it easier to use : func : ` pylab. gca ` without having to re - create the axes in the meantime.

         @param ax - the axis to set the current axes to. If you don't want to change the current axes use
        """
        if isinstance(ax, _mpl.axes._subplots.SubplotBase):
            return ax
        else:
            return _plt.gca()

    def orient(ax=None):
        """
         Orient the axes. This is a no - op if ax is None

         @param ax - axis to orient. If None a new axis is created
        """
        orient = "v"
        factor = 0
        axes = _axTools.get_axes(ax)
        rects = axes[0].patches
        if len(rects) > 0:
            for k, t in enumerate(axes):
                rects = t.patches
                if len(rects) > 1:
                    if not isinstance(rects[0], _mpl.patches.Wedge):
                        q = [round(x.get_width(), 2) for x in rects]
                        if all(x == q[0] for x in q):
                            orient = "v"
                            factor = q[0]
                        else:
                            orient = "h"
                            factor = rects[0].get_height()
                elif len(rects) == 1:
                    if not isinstance(rects[0], _mpl.patches.Wedge):
                        if rects[0].get_x() == 0:
                            orient = "h"
                        else:
                            orient = "v"
        return orient, factor

    def get_axes(ax=None):
        """
         Get or set the Axes object. This is a low - level function to allow you to get an Axes object from a list of axes or a single axes

         @param ax - If None ( default ) the current axes is returned
        """
        axes = []
        if isinstance(ax, _sns.axisgrid.FacetGrid):
            axes = ax.axes[0]
        elif isinstance(ax, list):
            axes = ax
        else:
            axes = [ax]
        return axes

    def getColorList(size: int, color):
        """
         Returns a list of colors that can be used to draw a color. The list is sorted by color and size

         @param size - the size of the color list
         @param color - the color to draw ( one of L { twisted. internet. colors. BGR }
        """
        colors = list()
        if type(color) is str:
            colors = [color] * size
        elif type(color) is list:
            miss = size - len(color)
            if miss < 0:
                colors = color[:size]
            elif miss > 0:
                colors = color
                for i in range(0, miss):
                    colors.append("white")
            else:
                colors = color
        return colors

    def values_bars(ax=None, normBy: str() = "c", loc: str() = "top", stack=False):
        """
         Plot values of a function. This is a wrapper around plot_bars that does not require matplotlib

         @param ax - axis to plot on.
         @param normBy - normalization method to use.
         @param loc - location to plot ( top bottom ). Default top
         @param stack - if True plot stack instead of normalizing. Default
        """
        pos = 0
        orient, factor = _axTools.orient(ax)
        data = _pd.DataFrame(
            columns=[
                "x",
                "y",
                "value",
                "value_norm",
                "index_color",
                "ax",
                "type",
                "color",
            ]
        )
        data = data.astype(
            {
                "x": "float32",
                "y": "float32",
                "value": "float32",
                "ax": "int32",
                "index_color": "int32",
            }
        )
        axes = _axTools.get_axes(ax)
        # Getting values
        for k, t in enumerate(axes):
            h = 0
            hist = {}
            rects = t.patches
            if len(rects) > 1:
                for rect in rects:
                    if not isinstance(rect, _mpl.patches.Wedge):
                        color = rect.get_fc()
                        if orient == "v":
                            value = (
                                0
                                if math.isnan(rect.get_height()) == True
                                else rect.get_height()
                            )
                            x = rect.get_x() + rect.get_width() / 2
                            if stack:
                                h = hist[x] if x in hist else 0
                            else:
                                h = 0
                            hist[x] = value + h
                            y = value + h
                        elif orient == "h":
                            value = (
                                0 if math.isnan(rect.get_width()) else rect.get_width()
                            )
                            y = rect.get_y() + rect.get_height() / 2
                            if stack:
                                h = hist[y] if y in hist else 0
                            else:
                                h = 0
                            hist[y] = value + h
                            x = value + h
                        data.loc[pos] = (x, y, value, 0, 0, int(k), "bar", color)
                        pos = pos + 1
            elif len(rects) == 1:
                rect = rects[0]
                if not isinstance(rect, _mpl.patches.Wedge):
                    color = rect.get_fc()
                    if orient == "h":
                        value = rect.get_width()
                        x = rect.get_width()
                        y = rect.get_x()
                    elif orient == "v":
                        value = rect.get_height()
                        y = rect.get_height()
                        x = rect.get_y()
                    data.loc[pos] = (x, y, value, 0, 0, int(k), "bar", color)

        # Grouping data by condition
        if normBy == "g":
            if isinstance(ax, _sns.axisgrid.FacetGrid):
                data.loc[:, "index_color"] = data.loc[:, "ax"]
            else:
                for i, color in enumerate(data["color"].unique()):
                    dt = data[data["color"] == color]
                    data.loc[dt.index, "index_color"] = i
        elif normBy == "c":
            if orient == "v":
                data.loc[:, "index_color"] = data.loc[:, "x"].round(0).astype("int")
            else:
                data.loc[:, "index_color"] = data.loc[:, "y"].round(0).astype("int")
        data.drop("color", axis=1, inplace=True)

        # Set position values
        vals = data.loc[:, "value"]
        vals = vals.apply(lambda x: x if x > 0 else -x)
        if loc == "center":
            vals = vals.apply(lambda x: x / 2)
        if orient == "v":
            if loc in ["bottom", "center"]:
                data.loc[:, "y"] = data.loc[:, "y"] - vals
        else:
            if loc in ["bottom", "center"]:
                data.loc[:, "x"] = data.loc[:, "x"] - vals
        return data

    def values_line(ax, orient):
        """
         Draws a line between values. This is a wrapper around : func : ` matplotlib. Axes. values ` that does not need to be implemented by subclasses

         @param ax - axis on which to draw
         @param orient - orientation of the line ( horizontal or vertical )
        """
        lnum = 0
        data = _pd.DataFrame(
            columns=["x", "y", "value", "value_norm", "index_color", "ax", "type"]
        )
        axes = _axTools.get_axes(ax)
        for l in axes[0].lines:
            if orient == "v":
                y = l.get_ydata()
                if not _np.isnan(y).any():
                    lnum += 1
                else:
                    x = l.get_xdata()
                    if not _np.isnan(x).any():
                        lnum += 1
        if lnum > 0:
            x_all = list()
            y_all = list()
            ax_all = list()
            color_all = list()

            for k, ax in enumerate(axes):
                lines = ax.lines
                if len(lines) > 0:
                    for j, l in enumerate(lines):
                        y = l.get_ydata()
                        ax_all.extend([k for x in y])
                        color_all.extend([j for x in y])
                        if all(isinstance(p, str) for p in y):
                            y = list(_np.arange(0, len(y)))
                        for i, txt in enumerate(y):
                            x = l.get_xdata()
                            if any(isinstance(p, str) for p in x):
                                x = list(_np.arange(0, len(x)))
                        if len(x) > 0 and len(y) > 0:
                            x_all.extend(x)
                            y_all.extend(y)
            # Set value location
            if orient == "h":
                data["x"] = x_all
                data["y"] = y_all
                data["value"] = x_all
            else:
                data["y"] = y_all
                data["x"] = x_all
                data["value"] = y_all
            # Set other columns values
            data["index_color"] = color_all
            data["ax"] = ax_all
            data["type"] = "line"
            data["value_norm"] = 0
        # Setting data types
        data = data.astype({"index_color": "int32", "ax": "int32"})
        data = data[_pd.notna(data["value"])].reset_index()
        return data.drop("index", axis=1)

    def values_points(ax):
        """
         Return a list of points that can be plotted. This is useful for plotting data that is not a plot but a series of plot points.

         @param ax - Axes on which to plot the data. If None the plot will be shown on the current axes
        """
        data = _pd.DataFrame(
            columns=["x", "y", "value", "value_norm", "index_color", "ax", "type"]
        )
        axes = _axTools.get_axes(ax)
        # Getting value points
        for i, h in enumerate(axes):
            d = h.collections
            for d1 in d:
                data1 = _pd.DataFrame()
                values = d1.get_offsets().tolist()
                if len(values) > 0:
                    rg = range(len(values))
                    data1["x"] = [v[0] for v in values]
                    data1["y"] = [v[1] for v in values]
                    data1["value"] = [v[1] for v in values]
                    data1["index_color"] = [0 for x in rg]
                    data1["ax"] = [i for x in rg]
                    data1["type"] = ["point" for x in rg]
                    data1["value_norm"] = 0
                    data = _pd.concat([data, data1])
        return data

    def plot_values(
        ax=None,
        data: _pd.DataFrame = None,
        minvalue: float = -np.Infinity,
        maxvalue: float = np.Infinity,
        dec: int = 2,
        fontsize=_mpl.rcParams["font.size"],
        fontweight: str = "normal",
        color="white",
        display: str = "v",
        prefix: str = "",
        postfix: str = "",
        orient: str = "v",
        **kwargs,
    ):
        """
        Plot values as a 2D plot. This is a wrapper around matplotlib. pyplot. plot_values that allows you to specify the minimum and maximum values to plot in one call.

        @param ax - matplotlib axis ( default : None ) If provided the plot will be added to this axis
        @param data - pandas DataFrame ( default : None ) If provided the plot
        @param minvalue
        @param dec
        @param fontsize
        @param fontweight
        @param color
        @param display
        @param prefix
        @param postfix
        @param orient
        """
        # Initial variables
        template = _axTools.template_print_value(display)
        va = kwargs["va"]
        ha = kwargs["ha"]
        if fontsize.isnumeric():
            fontsize = fontsize * 0.6 if fontsize > 10 else fontsize - 2
        axes = _axTools.get_axes(ax)

        shadow = kwargs["shadow"]
        shadowcolor = kwargs["shadowcolor"]
        _, kwargs = _axTools.get_init_kwargs(kwargs, get_d=False)

        # Check if is Digit function

        def isDigit(x):
            try:
                x = float(x)
                return True
            except ValueError:
                return False

        # Segment data with True values
        data = (
            data[data["value"].notnull()].copy().reset_index(inplace=False, drop=True)
        )
        colors = _axTools.getColorList(data.shape[0], color)
        # Setting data types
        if dec > 0:
            vector = data["value"].to_numpy()
            for i, row in enumerate(vector):
                if isinstance(row, (float)):
                    vector[i] = round(row, dec)
            data["value"] = vector
        else:
            data["value"] = data["value"].astype("int32")
        # Print values
        for k, g in enumerate(axes):
            data_sub = data[(data["ax"] == k)]
            for c, row in zip(colors, data_sub.to_numpy()):
                x = row[0]
                y = row[1]
                value = row[2]
                value_norm = round(row[3] * 100, dec)
                graph = row[6]

                valid_value = True
                if isDigit(value) and (minvalue is not None):
                    valid_value = (
                        True if (value >= minvalue and value <= maxvalue) else False
                    )

                if valid_value:
                    # Choosing graph type
                    if graph == "bar":
                        if orient == "v":
                            if va == "auto":
                                va_h = "bottom" if y >= 0 else "top"
                            else:
                                va_h = va
                            ha_h = "center" if ha == "auto" else ha

                        else:
                            if ha == "auto":
                                ha_h = "left" if x > 0 else "right"
                            else:
                                ha_h = ha
                            va_h = "center_baseline" if va == "auto" else va

                        text = g.text(
                            x,
                            y,
                            prefix
                            + template.substitute(value=value, value_norm=value_norm)
                            + postfix,
                            fontsize=fontsize,
                            color=c,
                            ha=ha_h,
                            va=va_h,
                            fontweight=fontweight,
                            **kwargs,
                        )
                    else:
                        ha_h = "center" if ha == "auto" else ha
                        va_h = "center_baseline" if va == "auto" else va

                        if graph == "line":
                            text = g.text(
                                x,
                                y,
                                prefix
                                + template.substitute(
                                    value=value, value_norm=value_norm
                                )
                                + postfix,
                                fontsize=fontsize,
                                color=c,
                                ha=ha_h,
                                va=va_h,
                                fontweight=fontweight,
                                **kwargs,
                            )
                        elif graph == "point":
                            text = g.text(
                                x,
                                y,
                                prefix + value + postfix,
                                fontsize=fontsize,
                                ha=ha_h,
                                va=va_h,
                                color=c,
                                **kwargs,
                            )
                    if shadow > 0:
                        text.set_path_effects(
                            [
                                path_effects.Stroke(
                                    linewidth=shadow, foreground=shadowcolor, alpha=0.8
                                ),
                                path_effects.Normal(),
                            ]
                        )

    def normalize_data(df: _pd.DataFrame = None):
        """
        Normalize data to make it easier to read. This is a no - op if data is already normalized

        @param df - pandas DataFrame
        """
        data = df.copy(deep=True)
        # Normalize data values
        grp = data.groupby(["type", "index_color"], as_index=False)["value"].sum()
        for row in data.itertuples():
            total = grp["value"][
                (grp["type"] == row.type) & (grp["index_color"] == row.index_color)
            ].values[0]
            data.loc[row.Index, "value_norm"] = data.loc[row.Index, "value"] / total
        return data

    def values_pie(ax, valid_wedges, frm, fontsize, dec, minvalue):
        """
        Plot pie chart values. This is a wrapper around plot_values.

        @param ax - Axes to plot on. Must have been created before.
        @param valid_wedges - List of valid edge positions.
        @param frm - Font to use for plot. If None defaults to font_size ( 1 ).
        @param fontsize - Font size in points. If None defaults to fontsize ( 1 ).
        @param dec - Decalization parameter passed to plot_values.
        @param minvalue - Minimum value to plot ( default 0 )
        """
        data = _pd.DataFrame()
        x = y = list()
        val_list = label_list = list()
        for i, p in enumerate(valid_wedges):
            val = p.theta2 - p.theta1
            ang = (p.theta2 - p.theta1) / 2.0 + p.theta1
            y.append(_np.sin(_np.deg2rad(ang)))
            x.append(_np.cos(_np.deg2rad(ang)) - 0.2)
            val_list.append((val / 360) * 100)
            label_list.append(p.get_label())
        data["x"] = x
        data["y"] = y
        data["val"] = val_list
        data["labels"] = label_list
        # Setting bbox style
        bbox_props = dict(
            boxstyle="round", ec=(0.1, 0.1, 0.1, 0.2), fc=(1.0, 1.0, 0.9, 0.75)
        )
        kw = dict(bbox=bbox_props, zorder=99, va="center")
        # Printing data
        for i in range(data.shape[0]):
            if data.val[i] > minvalue:
                value = frm % _np.round(data.val[i], dec) + "%"
                ax.annotate(
                    value,
                    xy=(data.x[i], data.y[i]),
                    fontsize=fontsize,
                    xytext=((0.75 * data.x[i]), (0.45 * data.y[i])),
                    **kw,
                )
        return data

    def template_print_value(display: str = str()):
        """
        @brief Prints the value of template_display. It is used to check if there is a value in template_display and if it is print it.
        @param display the value to print ( default : None )
        """
        if display == "f":
            template = Template("$value\n$value_norm%")
        elif display == "v":
            template = Template("$value")
        elif display == "p":
            template = Template("$value_norm%")
        else:
            raise ValueError(
                'The alternatives for display are "f=full", "v=values" or "p=percent"'
            )
        return template

    def get_init_kwargs(
        kwargs,
        remove_kwargs=True,
        get_d=True,
        inplace_kwargs=False,
        defaults: dict = {},
    ) -> tuple:
        """
        @brief Get kwargs to be used for initialization. This is a wrapper around : func : ` dict. get ` to allow the user to specify a set of keyword arguments that will be used to initialize the model.
        @param kwargs A dictionary of keyword arguments to be initialized.
        @param remove_kwargs If True the kwargs will be removed from the initial kwargs.
        @param get_d If True the dictionary will be returned as - is.
        @param inplace_kwargs If True the kwargs will be copied to the new dictionary instead of returned.
        @param defaults A dictionary of default values to be used for initialization.
        @return A tuple of the same length as the ` ` kwargs ` ` dictionary
        """
        keys = {
            "color": style.get_ticksColor(),
            "colors": style.get_ticksColor(),
            "grid_color": style.get_ticksColor(),
            "grid_alpha": 0.35,
            "fontweight": "normal",
            "fontsize": _mpl.rcParams["axes.labelsize"],
            "shadow": 0,
            "shadowcolor": style.get_axesColor(),
            "xpad": 0,
            "ypad": 0,
            "va": "top",
            "ha": "center",
        }

        for k, v in defaults.items():
            keys[k] = v

        dic_args = {}
        if get_d:
            values = list()
            for k, v in keys.items():
                values.append(kwargs.get(k, v))
            dic_args = dict(zip(keys, values))

        if remove_kwargs:
            for key in keys:
                if key in kwargs:
                    del kwargs[key]

        if inplace_kwargs:
            for k, v in dic_args.items():
                kwargs[k] = v

        return dic_args, kwargs
